@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -18,13 +18,12 @@
 from __future__ import division, print_function, absolute_import
 
 import math
-import struct
 from collections import namedtuple
 
 import hypothesis.internal.conjecture.utils as d
 from hypothesis.control import assume
-from hypothesis.internal.compat import int_to_bytes, int_from_bytes, \
-    bytes_from_list
+from hypothesis.internal.compat import hbytes, struct_pack, int_to_bytes, \
+    struct_unpack, int_from_bytes, bytes_from_list
 from hypothesis.internal.floats import sign
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
     MappedSearchStrategy
@@ -66,14 +65,13 @@ class WideRangeIntStrategy(IntStrategy):
 
         def distribution(random, n):
             assert n == size
-            k = min(
-                random.randint(0, n * 8 - 1),
-                random.randint(0, n * 8 - 1),
-            )
-            if k > 0:
-                r = random.getrandbits(k)
+            if random.randint(0, 2) > 0:
+                k = 1
+            elif random.randint(0, 2) > 0:
+                k = 2
             else:
-                r = 0
+                k = random.randint(1, size)
+            r = random.getrandbits(k * 8)
             if random.randint(0, 1):
                 r |= sign_mask
             else:
@@ -158,8 +156,8 @@ class FloatStrategy(SearchStrategy):
                         random.randint(-2 ** 63, 2 ** 63), 1
                     )
                 if self.permitted(f):
-                    return struct.pack(b'!d', f)
-        result = struct.unpack(b'!d', bytes(
+                    return struct_pack(b'!d', f)
+        result = struct_unpack(b'!d', hbytes(
             data.draw_bytes(8, draw_float_bytes)))[0]
         assume(self.permitted(result))
         return result
@@ -211,8 +209,8 @@ class FixedBoundedFloatStrategy(SearchStrategy):
                 f = random.random() * (
                     self.upper_bound - self.lower_bound
                 ) + self.lower_bound
-            return struct.pack(b'!d', f)
-        f = struct.unpack(b'!d', bytes(
+            return struct_pack(b'!d', f)
+        f = struct_unpack(b'!d', hbytes(
             data.draw_bytes(8, draw_float_bytes)))[0]
         assume(self.lower_bound <= f <= self.upper_bound)
         assume(sign(self.lower_bound) <= sign(f) <= sign(self.upper_bound))
