@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -25,7 +25,7 @@ from fractions import Fraction
 from hypothesis.errors import InvalidArgument
 from hypothesis.control import assume
 from hypothesis.searchstrategy import SearchStrategy
-from hypothesis.internal.compat import hrange, ArgSpec, text_type, \
+from hypothesis.internal.compat import ArgSpec, hrange, text_type, \
     getargspec, integer_types, float_to_decimal
 from hypothesis.internal.floats import is_negative, float_to_int, \
     int_to_float, count_between_floats
@@ -40,14 +40,14 @@ __all__ = [
     'choices', 'streaming',
     'booleans', 'integers', 'floats', 'complex_numbers', 'fractions',
     'decimals',
-    'characters', 'text', 'binary',
+    'characters', 'text', 'binary', 'uuids',
     'tuples', 'lists', 'sets', 'frozensets',
     'dictionaries', 'fixed_dictionaries',
     'sampled_from', 'permutations',
     'builds',
     'randoms', 'random_module',
     'recursive', 'composite',
-    'shared', 'runner',
+    'shared', 'runner', 'data',
 ]
 
 _strategies = set()
@@ -412,13 +412,13 @@ def sampled_from(elements):
 
     from hypothesis.searchstrategy.misc import SampledFromStrategy, \
         JustStrategy
-    elements = tuple(iter(elements))
+    from hypothesis.internal.conjecture.utils import check_sample
+    elements = check_sample(elements)
     if not elements:
         return nothing()
     if len(elements) == 1:
         return JustStrategy(elements[0])
-    else:
-        return SampledFromStrategy(elements)
+    return SampledFromStrategy(elements)
 
 
 @cacheable
@@ -468,7 +468,8 @@ def lists(
                 'want to set unique_by)'
             ))
         else:
-            unique_by = lambda x: x
+            def unique_by(x):
+                return x
 
     if unique_by is not None:
         from hypothesis.searchstrategy.collections import UniqueListStrategy
@@ -923,6 +924,7 @@ def composite(f):
 
                 return f(draw, *args, **kwargs)
         return CompositeStrategy()
+    accept.__module__ = f.__module__
     return accept
 
 
@@ -953,7 +955,7 @@ def choices():
 
     """
     from hypothesis.control import note, current_build_context
-    from hypothesis.internal.conjecture.utils import choice
+    from hypothesis.internal.conjecture.utils import choice, check_sample
 
     class Chooser(object):
 
@@ -965,7 +967,7 @@ def choices():
         def __call__(self, values):
             if not values:
                 raise IndexError('Cannot choose from empty sequence')
-            result = choice(self.data, values)
+            result = choice(self.data, check_sample(values))
             with self.build_context.local():
                 self.choice_count += 1
                 note('Choice #%d: %r' % (self.choice_count, result))
@@ -1037,7 +1039,8 @@ def data():
     It can only be used within @given, not find. This is because the lifetime
     of the object cannot outlast the test body.
 
-    See the rest of the documentation for more complete information.
+    See :ref:`the rest of the documentation <interactive-draw>` for more
+    complete information.
 
     """
     from hypothesis.control import note
